import javax.swing.JOptionPane;
import javax.swing.JDialog;
import javax.swing.JButton;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.BorderFactory;
import javax.swing.border.Border;
import javax.swing.JTabbedPane;
import javax.swing.JPanel;
import javax.swing.JFrame;
import java.beans.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Clase Dialogo
 * Clase que muestra en pantalla una ventana que permite modificar los
 * parametros del objeto escogido para seleccionar.
 * @author Patricio A. Castillo Pizarro - Eric C. Maldonado Olguin
 * @version 1.0 - Jun 26th, 2007
 */
public class Dialogo extends JPanel {
    JLabel label;
    ImageIcon icon = createImageIcon("images/middle.gif");
    JFrame frame;
    String simpleDialogDesc = "Parametro1";
    String iconDesc = "Parametro2";
    String moreDialogDesc = "Parametros del Objeto";
    DialogoModificar customDialog;
    public Forma forma;


    public Dialogo(JFrame frame, Forma f) {
        super(new BorderLayout());
        this.frame = frame;
        forma=f;
        customDialog = new DialogoModificar(frame, "Eric Maldonado", this);
        customDialog.pack();       
       
        JPanel featurePanel = createFeatureDialogBox();        
        label = new JLabel("Presione el Boton"
                           + " para Editar Parametros!",
                           JLabel.CENTER);

        //Lay them out.
        Border padding = BorderFactory.createEmptyBorder(20,20,5,20);      
        featurePanel.setBorder(padding);
        

        JTabbedPane tabbedPane = new JTabbedPane();
      
        tabbedPane.addTab("Parametros", null,
                          featurePanel,                         
                         moreDialogDesc);
        
        add(tabbedPane, BorderLayout.CENTER);
        add(label, BorderLayout.PAGE_END);
        label.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
    }

    void setLabel(String newText) {
        label.setText(newText);
    }

    protected static ImageIcon createImageIcon(String path) {
        java.net.URL imgURL = Dialogo.class.getResource(path);
        if (imgURL != null) {
            return new ImageIcon(imgURL);
        } else {
            System.err.println("No se encuenta archivo " + path);
            return null;
        }
    }

    private JPanel createPane(String description,
                              JRadioButton[] radioButtons,
                              JButton showButton) {

        int numChoices = radioButtons.length;
        JPanel box = new JPanel();
        JLabel label = new JLabel(description);

        box.setLayout(new BoxLayout(box, BoxLayout.PAGE_AXIS));
        box.add(label);

        for (int i = 0; i < numChoices; i++) {
            box.add(radioButtons[i]);
        }

        JPanel pane = new JPanel(new BorderLayout());
        pane.add(box, BorderLayout.PAGE_START);
        pane.add(showButton, BorderLayout.PAGE_END);
        return pane;
    }

     private JPanel create2ColPane(String description,
                                  JRadioButton[] radioButtons,
                                  JButton showButton) {
        JLabel label = new JLabel(description);
        int numPerColumn = radioButtons.length/2;

        JPanel grid = new JPanel(new GridLayout(0, 2));
        for (int i = 0; i < numPerColumn; i++) {
            grid.add(radioButtons[i]);
            grid.add(radioButtons[i + numPerColumn]);
        }

        JPanel box = new JPanel();
        box.setLayout(new BoxLayout(box, BoxLayout.PAGE_AXIS));
        box.add(label);
        grid.setAlignmentX(0.0f);
        box.add(grid);

        JPanel pane = new JPanel(new BorderLayout());
        pane.add(box, BorderLayout.PAGE_START);
        pane.add(showButton, BorderLayout.PAGE_END);

        return pane;
    }

   

    private JPanel createFeatureDialogBox() {
        final int numButtons = 4;
        JRadioButton[] radioButtons = new JRadioButton[numButtons];
        final ButtonGroup group = new ButtonGroup();

        JButton showItButton = null;

        final String pickOneCommand = "Frecuencia";
        final String textEnteredCommand = "Tiempo";
        final String nonAutoCommand = "Mensage";
        final String customOptionCommand = "Potencia";    
        
        
        radioButtons[0] = new JRadioButton("Frecuencia");
        radioButtons[0].setActionCommand(pickOneCommand);

        radioButtons[1] = new JRadioButton("Tiempo");
        radioButtons[1].setActionCommand(textEnteredCommand);
       

        radioButtons[2] = new JRadioButton("Mensage");
        radioButtons[2].setActionCommand(nonAutoCommand);

        radioButtons[3] = new JRadioButton("Potencia");
        radioButtons[3].setActionCommand(customOptionCommand);                        

        for (int i = 0; i < numButtons; i++) {
            group.add(radioButtons[i]);
        }
        radioButtons[0].setSelected(true);

        showItButton = new JButton("Modificar");
        showItButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String command = group.getSelection().getActionCommand();

               if (command == pickOneCommand) {
                    String s = (String)JOptionPane.showInputDialog(
                                        frame,
                                        "Ingrese Nuevo valor de Frecuencia:\n"
                                        ,
                                        "Customized Dialog",
                                        JOptionPane.PLAIN_MESSAGE,
                                        icon,
                                        null,
                                        "Numero Entero");

                    if ((s != null) && (s.length() > 0)) {
                        setLabel("Nuevo valor de Frecuencia : " + s);
                        forma.setFrecuencia(Integer.valueOf(s));
                        return;
                    }
                } else if (command == textEnteredCommand) {
                    String s = (String)JOptionPane.showInputDialog(
                                        frame,
                                        "Ingrese Nuevo valor de Tiempo:\n"
                                        ,
                                        "Customized Dialog",
                                        JOptionPane.PLAIN_MESSAGE,
                                        icon,
                                        null,
                                        "Numero Entero");

                    if ((s != null) && (s.length() > 0)) {
                        setLabel("Nuevo valor de Tiempo : " + s);
                        forma.setTiempo(Integer.valueOf(s));
                        return;
                    }

                 
              
                } else if (command == nonAutoCommand) {
                    String s = (String)JOptionPane.showInputDialog(
                                        frame,
                                        "Ingrese Nuevo valor de Mensage:\n"
                                        ,
                                        "Customized Dialog",
                                        JOptionPane.PLAIN_MESSAGE,
                                        icon,
                                        null,
                                        "Numero Entero");

                    if ((s != null) && (s.length() > 0)) {
                        setLabel("Nuevo valor de Mensage : " + s);
                        forma.setMessage(Integer.valueOf(s));
                        return;
                    }
                    
                    
                } else if (command == customOptionCommand) {
                    String s = (String)JOptionPane.showInputDialog(
                                        frame,
                                        "Ingrese Nuevo valor de Potencia:\n"
                                        ,
                                        "Customized Dialog",
                                        JOptionPane.PLAIN_MESSAGE,
                                        icon,
                                        null,
                                        "Numero Entero");

                    if ((s != null) && (s.length() > 0)) {
                        setLabel("Nuevo valor de Potencia sera : " + s);
                        forma.setPower(Integer.valueOf(s));
                        return;
                    }
                    
                    
                } 
                
                
                
            }
        });
        
       
       

        return createPane(moreDialogDesc + ":",
                          radioButtons,
                          showItButton);
    }

   
}

//Fin Clase Dialogo