import java.awt.event.*;
import java.awt.*;
import java.io.*;
import javax.swing.*;

/**
 * Clase MenuRobotFirmware
 * Permite cargar el firmware al RCX.
 * @author Patricio A. Castillo Pizarro - Eric C. Maldonado Olguin
 * @version 1.6 - Jun 24th, 2007
 */
class MenuRobotFirmware implements MouseListener
{
    private EditorPanel panel;
    
    /**
     * Constructor.
     * @param p Panel en que se encuentra el programa grafico a compilar
     */
    public MenuRobotFirmware(EditorPanel p)
    {
        panel = p;
    }
    
    /**
     *Metodo que define las acciones a realizar al presiona el boton del mouse
     *@param event Parametro que indica la accion realizada por el mouse
     */
    public void mousePressed(MouseEvent event){
        //Primero se verifica que este presente el compilador
        String compiler = new String();
        String sls = new String(System.getProperty("file.separator"));
        try{
            String path = new File(".").getCanonicalPath();
            compiler = path+sls+"nqc.exe";
            if (!new File(compiler).exists()){
                JOptionPane.showMessageDialog(null,
                        "No se encuentra el programa interfaz.\nAsegurese que "
                        +"\"nqc.exe\" se encuentra en la ruta desde la que se "
                        +"esta ejecutando esta aplicacion:\n"+path,
                        "Error", JOptionPane.ERROR_MESSAGE);
            }
        }catch (IOException e) {
            e.printStackTrace();
        }
        
        //Este codigo permite escoger el archivo
        JFileChooser fcOpen = new JFileChooser();
        fcOpen.setSelectedFile(new File("*.lgo"));
        fcOpen.setFileFilter(new FileFilterLGO());
        int return_fcOpen = fcOpen.showSaveDialog(null);
        File firmware = fcOpen.getSelectedFile();
        
        //Este codigo permite enviar el firmware escogido al robot
        if (return_fcOpen == JFileChooser.APPROVE_OPTION) {
            if (new File(compiler).exists()) {
                boolean hecho = true;
                String command ="\""+compiler+"\" -T"+panel.tipo+" -S"+
                        panel.puerto+" -firmware \""+
                        firmware.getAbsolutePath()+"\"";
                Runtime run = Runtime.getRuntime();
                try{
                    String line;
                    panel.panelOUT = new String();
                    BufferedReader input = new BufferedReader(
                            new InputStreamReader(
                            run.exec(command).getInputStream()));
                    while ((line = input.readLine()) != null) {
                        System.out.println(line);
                        panel.panelOUT = panel.panelOUT + line;
                        hecho = false;
                    }
                    input.close();
                    if (!hecho){
                        System.out.println("No se pueden enviar los datos");
                        JOptionPane.showMessageDialog(null,
                                "No se pueden enviar los datos.\nVerifique el"+
                                " archivo seleccionado y robot.",
                                "Error", JOptionPane.ERROR_MESSAGE);
                    }
                } catch (Exception e){
                    System.out.println("No se pueden enviar los datos");
                    JOptionPane.showMessageDialog(null,
                            "No se pueden enviar los datos.\nVerifique el"+
                            " archivo seleccionado y el programa interfaz.",
                            "Error", JOptionPane.ERROR_MESSAGE);
                }
            }
        }else{
            JOptionPane.showMessageDialog(null,
                    "Ningun archivo seleccionado.\nNo se cargara firmware.",
                    "Advertencia", JOptionPane.WARNING_MESSAGE);
        }
    }
    
    /**
     *Implementacion de MouseListener cuando el boton del mouse hace click
     */
    public void mouseClicked(MouseEvent event){}

    /**
     *Implementacion de MouseListener cuando el boton del mouse se libera
     */
    public void mouseReleased(MouseEvent event){}

    /**
     *Implementacion de MouseListener cuando el mouse ingresa a un componente
     */
    public void mouseEntered(MouseEvent event){}

    /**
     *Implementacion de MouseListener cuando el mouse sale de un componente
     */
    public void mouseExited(MouseEvent event){}
}
//Fin clase MenuRobotFirmware
